part of '../../../../moru_payment_gateway.dart';

class MoRuPaymentCard extends StatefulWidget {
  final num amount;
  final ValidatedAccessRes validatedAccessRes;
  final String accessKey;

  const MoRuPaymentCard(
      {Key? key,
      required this.validatedAccessRes,
      required this.accessKey,
      required this.amount})
      : super(key: key);

  @override
  _MoRuPaymentCardState createState() => _MoRuPaymentCardState();
}

class _MoRuPaymentCardState extends State<MoRuPaymentCard> {
  //cubits
  late final UserValidatorCubit _userValidatorCubit;
  late final InitiatePayCubit _initiatePayCubit;
  late final ResendOtpTimerCubit _resendOtpTimerCubit;

  //bool
  bool _showPin = true;
  bool hasInvalidPin = false;
  bool isUserValidated = false;
  bool isUserValidatorLoading = false;
  bool isInitiatePayLoading = false;
  TextEditingController _otpController = TextEditingController();

  //string

  String? invalidRequestMessage;

  //validatedUserRes
  ValidatedUserRes? _validatedUserRes;

  //form
  final _formKey = GlobalKey<FormState>();

  int? phone;
  int? pin;
  int? otp;

  @override
  void initState() {
    super.initState();
    _userValidatorCubit = UserValidatorCubit();
    _initiatePayCubit = InitiatePayCubit();
    _resendOtpTimerCubit = ResendOtpTimerCubit();
  }

  @override
  void dispose() {
    super.dispose();
    _initiatePayCubit.close();
    _userValidatorCubit.close();
    _resendOtpTimerCubit.close();
  }

  @override
  Widget build(BuildContext context) {
    return MultiBlocListener(
      listeners: [
        BlocListener<UserValidatorCubit, UserValidatorState>(
            bloc: _userValidatorCubit,
            listener: (context, state) {
              if (state is UserValidatorLoading) {
                setState(() {
                  isUserValidatorLoading = true;
                });
              }
              if (state is UserValidatorSuccess) {
                _validatedUserRes = state.validatedUserRes;
                if (_validatedUserRes != null &&
                    _validatedUserRes!.data!.otp != null) {
                  _otpController.text = _validatedUserRes!.data!.otp.toString();
                }
                if (_validatedUserRes != null &&
                    _validatedUserRes!.data!.moruTxnIdentifier != null) {
                  setState(() {
                    isUserValidated = true;
                    isUserValidatorLoading = false;
                    _resendOtpTimerCubit.activeCounter();
                  });
                }
              } else if (state is UserValidatorFail) {
                setState(() {
                  isUserValidatorLoading = false;
                });
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    backgroundColor: MoruTheme.primaryColor,
                    content: Text(state.failedReason),
                  ),
                );
              }
            }),
        BlocListener<InitiatePayCubit, InitiatePayState>(
            bloc: _initiatePayCubit,
            listener: (context, state) {
              if (state is InitiatePayLoading) {
                setState(() {
                  isInitiatePayLoading = true;
                });
              }
              if (state is PaySuccess) {
                setState(() {
                  isInitiatePayLoading = false;
                });
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    backgroundColor: Colors.green,
                    content: Text("Your payment is successful"),
                  ),
                );
                Future.delayed(Duration(microseconds: 200)).then((value) =>
                    Navigator.of(context).pop(CheckoutStatus.COMPLETED));
              } else if (state is PayFail) {
                setState(() {
                  isInitiatePayLoading = false;
                });
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    backgroundColor: Colors.red,
                    content: Text(state.message),
                  ),
                );
                // Future.delayed(Duration(microseconds: 200)).then((value) =>
                //     Navigator.of(context).pop(CheckoutStatus.FAILED));
              }
            }),
      ],
      child: Scaffold(
        appBar: AppBar(
          title: Text("Moru Payment Card"),
          backgroundColor: MoruTheme.primaryColor,
        ),
        body: SingleChildScrollView(
          child: Padding(
            padding: const EdgeInsets.all(12.0),
            child: Form(
              key: _formKey,
              child: Column(
                children: [
                  Image.asset(
                    'assets/images/logo.png',
                    width: 100,
                    filterQuality: FilterQuality.high,
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Container(
                    height: 50,
                    color: Colors.grey[200],
                    child: Center(
                      child: Text(
                        "Total Amount: Rs.${widget.amount}",
                        style: TextStyle(
                            fontWeight: FontWeight.w500, fontSize: 16),
                      ),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Visibility(
                    visible: !isUserValidated,
                    child: TextFormField(
                      autovalidateMode: AutovalidateMode.onUserInteraction,
                      onSaved: (mobileNo) {
                        if (mobileNo != null && mobileNo.isNotEmpty)
                          phone = int.parse(mobileNo);
                      },
                      validator: (phone) {
                        return validateMobile(phone);
                      },
                      decoration: FieldDecoration.fieldDecoration(
                          context: context,
                          errorMessage: null,
                          labelText: 'Moru Mobile Number'),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Visibility(
                    visible: !isUserValidated,
                    child: TextFormField(
                      validator: (value) {
                        if (value != null && value.isNotEmpty) {
                          return null;
                        }
                        return 'Please enter Moru Pin';
                      },
                      autovalidateMode: AutovalidateMode.onUserInteraction,
                      onSaved: (moruPin) {
                        if (moruPin != null && moruPin.isNotEmpty)
                          pin = int.parse(moruPin);
                      },
                      maxLengthEnforcement: MaxLengthEnforcement.enforced,
                      maxLength: 4,
                      keyboardType: TextInputType.number,
                      obscureText: _showPin,
                      decoration: InputDecoration(
                        errorText: hasInvalidPin ? 'Invalid  Pin' : null,
                        errorBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(8),
                          borderSide: BorderSide(
                            width: 0.1,
                            color: Theme.of(context).errorColor,
                            style: BorderStyle.solid,
                          ),
                        ),
                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(8),
                          borderSide: BorderSide(
                            width: 0.1,
                            color: MoruTheme.primaryColor,
                            style: BorderStyle.solid,
                          ),
                        ),
                        border: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(8),
                          borderSide: BorderSide(
                            width: 0.1,
                            color: Colors.transparent,
                            style: BorderStyle.solid,
                          ),
                        ),
                        suffixIcon: IconButton(
                          icon: Icon((_showPin)
                              ? Icons.visibility
                              : Icons.visibility_off),
                          onPressed: () => this.setState(() {
                            _showPin = !_showPin;
                          }),
                        ),
                        contentPadding: EdgeInsets.all(8.0),
                        labelText: 'Moru PIN',
                      ),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Visibility(
                    visible: isUserValidated,
                    child: TextFormField(
                      controller: _otpController,
                      onSaved: (enteredOtp) {
                        if (enteredOtp != null && enteredOtp.isNotEmpty)
                          otp = int.parse(enteredOtp);
                      },
                      validator: (value) {
                        if (value != null && value.isNotEmpty) {
                          return null;
                        }
                        return 'Please enter OTP';
                      },
                      keyboardType: TextInputType.number,
                      maxLengthEnforcement: MaxLengthEnforcement.enforced,
                      maxLength: 6,
                      decoration: FieldDecoration.fieldDecoration(
                          context: context,
                          errorMessage: null,
                          labelText: 'OTP'),
                    ),
                  ),
                  Visibility(
                    visible: isUserValidated,
                    child: Center(
                      child:
                          BlocBuilder<ResendOtpTimerCubit, ResendOtpTimerState>(
                        bloc: _resendOtpTimerCubit,
                        builder: (context, state) {
                          if (state is RecordTimer) {
                            return Container(
                              width: 300,
                              height: 50,
                              child: TextButton(
                                child: Center(
                                  child: state.time == 0
                                      ? Text(
                                          'Resend code',
                                          style: TextStyle(
                                              color: MoruTheme.primaryColor),
                                        )
                                      : Row(
                                          mainAxisAlignment:
                                              MainAxisAlignment.center,
                                          children: <Widget>[
                                            Text(
                                                'You can request next OTP in ${state.time > 0 ? state.time.toString() : 30} seconds '),
                                          ],
                                        ),
                                ),
                                onPressed: state.time == 0
                                    ? () {
                                        _userValidatorCubit.validateUser(
                                          userValidate: UserValidate(
                                            accessKey: widget.accessKey,
                                            moruTxnIdentifier: widget
                                                .validatedAccessRes
                                                .accessData!
                                                .moruTxnIdentifier!,
                                            pin: pin!,
                                            mobileNo: phone!,
                                          ),
                                        );
                                      }
                                    : null,
                              ),
                            );
                          }

                          return SizedBox.shrink();
                        },
                      ),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Visibility(
                    visible: !isUserValidated,
                    child: MaterialButton(
                      color: MoruTheme.primaryColor,
                      textColor: Colors.black,
                      minWidth: MediaQuery.of(context).size.width,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(7),
                      ),
                      height: 48,
                      highlightColor: Colors.white30,
                      elevation: 0,
                      highlightElevation: 0,
                      onPressed: () {
                        if (_formKey.currentState!.validate()) {
                          _formKey.currentState!.save();
                          _userValidatorCubit.validateUser(
                            userValidate: UserValidate(
                                accessKey: widget.accessKey,
                                moruTxnIdentifier: widget.validatedAccessRes
                                    .accessData!.moruTxnIdentifier!,
                                pin: pin!,
                                mobileNo: phone!),
                          );
                        }
                      },
                      padding: EdgeInsets.all(12),
                      child: isUserValidatorLoading
                          ? CircularProgressIndicator(color: Colors.white)
                          : Text(
                              "Continue",
                              style: TextStyle(
                                  fontWeight: FontWeight.w800,
                                  color: Colors.white,
                                  fontSize: 14),
                            ),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                  Visibility(
                    visible: isUserValidated,
                    child: MaterialButton(
                      color: MoruTheme.primaryColor,
                      textColor: Colors.black,
                      minWidth: MediaQuery.of(context).size.width,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(7),
                      ),
                      height: 48,
                      highlightColor: Colors.white30,
                      onPressed: () {
                        print("FINAL OTP" +
                            _validatedUserRes!.data!.otp!.toString());
                        if (_formKey.currentState!.validate()) {
                          _formKey.currentState!.save();
                          print(_formKey);
                          _initiatePayCubit.initiatePay(
                            payData: PayData(
                                accessKey: widget.accessKey,
                                moruTxnIdentifier:
                                    _validatedUserRes!.data!.moruTxnIdentifier!,
                                otp: otp!),
                          );
                        }
                      },
                      padding: EdgeInsets.all(12),
                      child: isInitiatePayLoading
                          ? CircularProgressIndicator(color: Colors.white)
                          : Text(
                              "Pay",
                              style: TextStyle(
                                  fontWeight: FontWeight.w800,
                                  color: Colors.white,
                                  fontSize: 14),
                            ),
                    ),
                  ),
                  const SizedBox(
                    height: 20,
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  String? validateMobile(String? phone) {
    String pattern = r'(^(?:[+0]9)?[0-9]{10,12}$)';
    RegExp regExp = new RegExp(pattern);
    if (phone != null && phone.isNotEmpty) {
      if (!regExp.hasMatch(phone)) {
        return 'Please enter valid mobile number';
      }
      return null;
    }
    return 'Please enter mobile number';
  }
}
