part of '../../../../moru_payment_gateway.dart';

class MoruSDKLauncher extends StatefulWidget {
  final CheckoutOptions checkoutOptions;
  final Function(CheckoutStatus status, ValidatedAccessRes accessRes) checkoutListener;

  const MoruSDKLauncher(
      {Key? key, required this.checkoutOptions, required this.checkoutListener})
      : super(key: key);

  @override
  _MoruSDKLauncherState createState() => _MoruSDKLauncherState();
}

class _MoruSDKLauncherState extends State<MoruSDKLauncher> {
  late CheckoutCubit _checkoutCubit;

  @override
  void initState() {
    super.initState();
    _checkoutCubit = CheckoutCubit(accessKey: widget.checkoutOptions.accessKey)
      ..initiatePay(checkoutOptions: widget.checkoutOptions);
  }

  @override
  void dispose() {
    super.dispose();
    _checkoutCubit.close();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: BlocListener<CheckoutCubit, CheckoutState>(
        bloc: _checkoutCubit,
        listener: (context, state) {
          if (state is ValidateAccessLoading) {
            showDialog(
                context: context,
                builder: (context) {
                  return Center(
                    child: Column(
                      children: [
                        SizedBox(
                            height: 20,
                            width: 20,
                            child: CircularProgressIndicator()),
                      ],
                    ),
                  );
                });
          }
          if (state is ValidateAccessSuccess) {
            if (state.accessRes.accessData != null) {
              _proceedCheckoutFlow(accessRes: state.accessRes);
            }
          } else if (state is ValidateAccessFail) {
            Navigator.of(context).pop();
            ScaffoldMessenger.of(context).showSnackBar(
              SnackBar(
                backgroundColor: MoruTheme.primaryColor,
                content: Text(state.failedReason),
              ),
            );
          }
        },
        child: Center(
          child: CircularProgressIndicator(),
        ),
      ),
    );
  }

  void _proceedCheckoutFlow({required ValidatedAccessRes accessRes}) async {
    Navigator.of(context).pop();
    final CheckoutStatus _checkoutStatus = await Navigator.of(context).push(
      MaterialPageRoute(
        builder: (context) => MoRuPaymentCard(
            amount: widget.checkoutOptions.amount,
            accessKey: widget.checkoutOptions.accessKey,
            validatedAccessRes: accessRes),
      ),
    );
    widget.checkoutListener(_checkoutStatus, accessRes);
  }
}
